data = {
    "g++": {
        "platform": "host",
        "projects": [
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["any"],
                "changelog": "https://gcc.gnu.org/",
                "prereqs": ["gcc"],
                "checker": {
                    "type": "pacman",
                    "package": "gcc",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
            {
                "arch": ["any"],
                "os": ["oraclelinux"],
                "osVersion": ["7"],
                "abi": ["any"],
                "changelog": "https://gcc.gnu.org/",
                "checker": {
                    "type": "yum",
                    "package": "gcc-c++",
                },
                "installer": {
                    "type": "yum",
                    "package": "gcc-c++",
                },
            },
            {
                "arch": ["any"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["any"],
                "changelog": "https://gcc.gnu.org/",
                "prereqs": [
                    "gcc",
                ],
                "checker": {
                    "type": "apt-cache",
                    "package": "gcc-12-monolithic",
                },
                "installer": {
                    "type": "cmd",
                    "cmds": [
                        [
                            "update-alternatives", "--install", "/usr/bin/g++",
                            "g++", "/usr/bin/g++-12", "1"
                        ],
                    ],
                },
            },
        ],
    },
    "gcc": {
        "platform": "host",
        "projects": [
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["any"],
                "changelog": "https://gcc.gnu.org/",
                "checker": {
                    "type": "pacman",
                    "package": "gcc",
                },
                "installer": {
                    "type": "pacman",
                    "package": "gcc",
                },
            },
            {
                "arch": ["any"],
                "os": ["oraclelinux"],
                "osVersion": ["7"],
                "abi": ["gnu"],
                "changelog": "https://gcc.gnu.org/",
                "checker": {
                    "type": "yum",
                    "package": "gcc",
                },
                "installer": {
                    "type": "yum",
                    "package": "gcc",
                },
            },
            {
                "arch": ["any"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["any"],
                "changelog": "https://gcc.gnu.org/",
                "checker": {
                    "type": "apt-cache",
                    "package": "gcc-12-monolithic",
                },
                "installer": {
                    "type": "apt",
                    "package": "gcc-12-monolithic",
                    "postInstallCommands": [
                        [
                            "update-alternatives", "--install", "/usr/bin/cpp",
                            "cpp", "/usr/bin/cpp-12", "1"
                        ],
                        [
                            "update-alternatives", "--set", "/usr/bin/cpp",
                            "cpp-bin", "/usr/bin/cpp-12", "1"
                        ],
                        [
                            "update-alternatives", "--install", "/usr/bin/gcc",
                            "gcc", "/usr/bin/gcc-12", "1"
                        ],
                        [
                            "update-alternatives", "--install",
                            "/usr/bin/gcc-ar", "gcc-ar", "/usr/bin/gcc-ar-12",
                            "1"
                        ],
                        [
                            "update-alternatives", "--install",
                            "/usr/bin/gcc-nm", "gcc-nm", "/usr/bin/gcc-nm-12",
                            "1"
                        ],
                        [
                            "update-alternatives", "--install",
                            "/usr/bin/gcc-ranlib", "gcc-ranlib",
                            "/usr/bin/gcc-ranlib-12", "1"
                        ],
                        [
                            "update-alternatives", "--install",
                            "/usr/bin/gcov", "gcov", "/usr/bin/gcov-12", "1"
                        ]
                    ],
                },
            },
        ],
    },
    "glu": {
        "platform": "target",
        "projects": [
            {
                "arch": ["any"],
                "os": ["linux"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["sgi-b-1.1", "sgi-b-2.0"],
                "changelog": "https://gitlab.freedesktop.org/mesa/glu/-/tags",
                "prereqs": [
                    "{triplet}-gcc",
                    "{triplet}-g++",
                    "{triplet}-pkg-config",
                    "meson",
                    "ninja",
                    "opengl",
                ],
                "checker": {
                    "type": "git",
                    "url": "https://gitlab.freedesktop.org/mesa/glu.git",
                },
                "installer": {
                    "type": "custom",
                    "name": "mesa-glu",
                },
            },
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["gnu"],
                "license": ["sgi-b-1.1", "sgi-b-2.0"],
                "changelog": "https://gitlab.freedesktop.org/mesa/glu/-/tags",
                "checker": {
                    "type": "pacman",
                    "package": "glu",
                },
                "installer": {
                    "type": "pacman",
                    "package": "glu",
                },
            },
            {
                "arch": ["any"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["gnu"],
                "license": ["sgi-b-1.1", "sgi-b-2.0"],
                "changelog": "https://gitlab.freedesktop.org/mesa/glu/-/tags",
                "checker": {
                    "type": "apt-cache",
                    "package": "libglu1-mesa-dev",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
        ],
    },
    "hash_table": {
        "platform": "target",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["mit"],
                "changelog": "https://github.com/anholt/hash_table/commits/master/",
                "prereqs": [
                    "{triplet}-gcc",
                ],
                "checker": {
                    "type": "git",
                    "url": "https://github.com/anholt/hash_table.git",
                    "files": ["meson.build"],
                    "hint": r'\d\.\d',
                },
                "installer": {
                    "type": "custom",
                    "name": "hash_table",
                },
            },
        ],
    },
    "help2man": {
        "platform": "host",
        "projects": [
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["any"],
                "changelog": "https://savannah.gnu.org/news/?group=coreutils",
                "checker": {
                    "type": "pacman",
                    "package": "help2man",
                },
                "installer": {
                    "type": "pacman",
                    "package": "help2man",
                },
            },
            {
                "arch": ["any"],
                "os": ["oraclelinux"],
                "osVersion": ["7"],
                "abi": ["any"],
                "changelog": "https://savannah.gnu.org/news/?group=coreutils",
                "checker": {
                    "type": "yum",
                    "repo": "ol7_optional_latest",
                    "package": "help2man",
                },
                "installer": {
                    "type": "yum",
                    "repo": "ol7_optional_latest",
                    "package": "help2man",
                },
            },
            {
                "arch": ["any"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["any"],
                "changelog": "https://savannah.gnu.org/news/?group=coreutils",
                "checker": {
                    "type": "apt-cache",
                    "package": "help2man",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
        ],
    },
    "icu": {
        "platform": "target",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["unicode", "icu", "bsd-3", "naist", "bsd-2"],
                "changelog": "https://github.com/unicode-org/icu/releases",
                "prereqs": [
                    "{triplet}-gcc",
                    "{triplet}-g++",
                    "g++",
                    "gcc",
                    "make",
                ],
                "checker": {
                    "type": "git",
                    "url": "https://github.com/unicode-org/icu.git",
                    "hint": r'release-\d+-\d$',
                },
                "installer": {
                    "type": "custom",
                    "name": "icu",
                },
            },
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["gnu"],
                "license": ["unicode", "icu", "bsd-3", "naist", "bsd-2"],
                "changelog": "https://github.com/unicode-org/icu/releases",
                "checker": {
                    "type": "pacman",
                    "package": "icu",
                },
                "installer": {
                    "type": "pacman",
                    "package": "icu",
                },
            },
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["oraclelinux"],
                "osVersion": ["7"],
                "abi": ["gnu"],
                "license": ["unicode", "icu", "bsd-3", "naist", "bsd-2"],
                "changelog": "https://github.com/unicode-org/icu/releases",
                "prereqs": [
                    "{triplet}-gcc",
                    "{triplet}-g++",
                    "g++",
                    "gcc",
                    "make",
                ],
                "checker": {
                    "type": "fixed",
                    "version": "67.1",
                },
                "installer": {
                    "type": "custom",
                    "name": "icu-67",
                },
            },
            {
                "arch": ["any"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["gnu"],
                "license": ["unicode", "icu", "bsd-3", "naist", "bsd-2"],
                "changelog": "https://github.com/unicode-org/icu/releases",
                "checker": {
                    "type": "apt-cache",
                    "package": "libicu-dev",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
        ],
    },
    "icu-54.2": {
        "platform": "target",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["unicode", "icu", "bsd-3", "naist", "bsd-2"],
                "changelog": "https://github.com/unicode-org/icu/releases",
                "prereqs": [
                    "{triplet}-gcc",
                    "{triplet}-g++",
                    "g++",
                    "gcc",
                    "make",
                ],
                "checker": {
                    "type": "fixed",
                    "version": "54.2",
                },
                "installer": {
                    "type": "custom",
                    "name": "icu_54_2",
                },
            },
        ],
    },
    "inih": {
        "platform": "target",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["bsd-3"],
                "changelog": "https://github.com/benhoyt/inih/releases",
                "prereqs": [
                    "{triplet}-gcc",
                ],
                "checker": {
                    "type": "git",
                    "url": "https://github.com/benhoyt/inih.git",
                },
                "installer": {
                    "type": "custom",
                    "name": "inih",
                },
            },
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["gnu"],
                "license": ["bsd-3"],
                "changelog": "https://github.com/benhoyt/inih/releases",
                "checker": {
                    "type": "pacman",
                    "package": "libinih",
                },
                "installer": {
                    "type": "pacman",
                    "package": "libinih",
                },
            },
            # Temporary stub
            {
                "arch": ["any"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["gnu"],
                "license": ["bsd-3"],
                "changelog": "https://github.com/benhoyt/inih/releases",
                "checker": {
                    "type": "apt-cache",
                    "package": "libinih-dev",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
        ],
    },
    "iwyu": {
        "platform": "host",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["bsd-3"],
                "changelog": "https://include-what-you-use.org/",
                "prereqs": [
                    "clang",
                    "cmake",
                    "g++",
                    "llvm",
                    "make",
                ],
                "checker": {
                    "type": "git",
                    "url": "https://github.com/include-what-you-use/include-what-you-use.git",
                    "hint": r'\d.\d+',
                },
                "installer": {
                    "type": "custom",
                    "name": "iwyu",
                },
            },
        ],
    },
}



import os

from libvgazer.command     import RunCommand
from libvgazer.exceptions  import CommandError
from libvgazer.exceptions  import InstallError
from libvgazer.platform    import GetInstallPrefix
from libvgazer.store.temp  import StoreTemp
from libvgazer.version.git import GetLastTag
from libvgazer.working_dir import WorkingDir

def Install(software, platform, platformData, verbose):
    installPrefix = GetInstallPrefix(platformData)
    storeTemp = StoreTemp()
    storeTemp.ResolveEmptySubdirectory(software)
    tempPath = storeTemp.GetSubdirectoryPath(software)

    try:
        with WorkingDir(tempPath):
            RunCommand(
             [
              "git", "clone",
              "https://gitlab.freedesktop.org/xorg/util/macros.git", "."
             ],
             verbose)
            RunCommand(
             [
              "git", "checkout",
              GetLastTag("https://gitlab.freedesktop.org/xorg/util/macros.git",
               hint=r'util-macros-1\.\d\d\.\d')
             ],
             verbose)
            RunCommand(["./autogen.sh",
             "--prefix={prefix}".format(prefix=installPrefix),
             "PKG_CONFIG_PATH={prefix}/lib/pkgconfig:"
             "{prefix}/share/pkgconfig".format(prefix=installPrefix)], verbose)
            RunCommand(["make", "install"], verbose)
    except CommandError:
        print("VGAZER: Unable to install", software)
        raise InstallError(software + " not installed")

    print("VGAZER:", software, "installed")



import os

from libvgazer.command     import RunCommand
from libvgazer.env_vars    import EnvVar
from libvgazer.exceptions  import CommandError
from libvgazer.exceptions  import InstallError
from libvgazer.platform    import GetInstallPrefix
from libvgazer.store.temp  import StoreTemp
from libvgazer.version.git import GetLastTag
from libvgazer.working_dir import WorkingDir

def Install(software, platform, platformData, verbose):
    storeTemp = StoreTemp()
    storeTemp.ResolveEmptySubdirectory(software)
    tempPath = storeTemp.GetSubdirectoryPath(software)

    installPrefix = GetInstallPrefix(platformData)

    if "PKG_CONFIG_PATH" in os.environ:
        pkg_config_path = "{prefix}/lib/pkgconfig:{old}".format(
         prefix=installPrefix, old=os.environ["PKG_CONFIG_PATH"])
    else:
        pkg_config_path = "{prefix}/lib/pkgconfig".format(prefix=installPrefix)

    try:
        with WorkingDir(tempPath), EnvVar("PKG_CONFIG_PATH", pkg_config_path):
            RunCommand(
             [
              "git", "clone",
              "https://gitlab.freedesktop.org/wayland/wayland.git", "."
             ],
             verbose)
            RunCommand(
             [
              "git", "checkout",
              GetLastTag("https://gitlab.freedesktop.org/wayland/wayland.git",
               hint=r'1\.\d\d.\d+')
             ],
             verbose)
            RunCommand(
             [
              "meson", "setup", "build/", "--prefix=/usr/local",
              "-Dlibraries=false", "-Dtests=false", "-Ddocumentation=false",
              "-Ddtd_validation=false"
             ],
             verbose)
            RunCommand(["ninja", "-C", "build/"], verbose)
            RunCommand(["ninja", "-C", "build/", "install"], verbose)
    except CommandError:
        print("VGAZER: Unable to install", software)
        raise InstallError(
         "{software} not installed".format(software=software))

    print("VGAZER:", software, "installed")



from libvgazer.command    import RunCommand
from libvgazer.exceptions import CommandError
from libvgazer.exceptions import InstallError

def InstallCmd(software, cmds, verbose):
    for cmd in cmds:
        try:
            RunCommand(cmd, verbose)
        except CommandError:
            print("VGAZER: Unable to install", software)
            raise InstallError(
             "{software}  not installed".format(software=software))

    print("VGAZER:", software, "installed")



import importlib

from libvgazer.exceptions import MissingChecker

class VersionCustom:
    def __init__(self, customCheckers={}):
        self.customCheckers = customCheckers

    def AddData(self, customCheckers):
        self.customCheckers = {**self.customCheckers, **customCheckers}

    def Check(self, checker):
        try:
            versionChecker = importlib.import_module(
             'libvgazer.version.custom_checker.' + checker)
        except ImportError:
            if len(self.customCheckers) == 0:
                raise MissingChecker(
                 "Missing custom checker : " + checker)
            for customChecker in self.customCheckers:
                if customChecker["name"] == checker:
                    versionChecker = customChecker["checker"]

        return versionChecker.Check()



class CommandError(Exception):
    def __init__(self, message, command, errorcode):
        super().__init__(message)
        self.command = command
        self.errorcode = errorcode

class CompatibleProjectNotFound(Exception):
    def __init__(self, message):
        super().__init__(message)

class DirNameEngaged(Exception):
    def __init__(self, message):
        super().__init__(message)

class FilenameEngaged(Exception):
    def __init__(self, message):
        super().__init__(message)

class FileNotFound(Exception):
    def __init__(self, message):
        super().__init__(message)

class InstallError(Exception):
    def __init__(self, message):
        super().__init__(message)

class MissingArgument(Exception):
    def __init__(self, message):
        super().__init__(message)

class MissingChecker(Exception):
    def __init__(self, message):
        super().__init__(message)

class MissingInstaller(Exception):
    def __init__(self, message):
        super().__init__(message)

class OsDataNotFound(Exception):
    def __init__(self, message):
        super().__init__(message)

class ReleaseDataNotFound(Exception):
    def __init__(self, message):
        super().__init__(message)

class TarballLost(Exception):
    def __init__(self, message):
        super().__init__(message)

class UnexpectedOsType(Exception):
    def __init__(self, message):
        super().__init__(message)

class UnknownInstaller(Exception):
    def __init__(self, message):
        super().__init__(message)

class UnknownOs(Exception):
    def __init__(self, message):
        super().__init__(message)

class UnknownOsVersion(Exception):
    def __init__(self, message):
        super().__init__(message)

class UnknownSoftware(Exception):
    def __init__(self, message):
        super().__init__(message)

class UnknownTargetArch(Exception):
    def __init__(self, message):
        super().__init__(message)

class UnknownVersionChecker(Exception):
    def __init__(self, message):
        super().__init__(message)



import os
from libvgazer.platform import GetHomeDirectoryPath
from libvgazer.platform import GetHideDirectoryPrefix
from libvgazer.store.base import StoreBase

class StoreHome(StoreBase):
    def __init__(self, subdirectory=None):
        homeDirPath = GetHomeDirectoryPath()
        if subdirectory is None:
            subdirectory = GetHideDirectoryPrefix() + "vgazer"
        super().__init__(os.path.join(homeDirPath, subdirectory))



from libvgazer.command import GetCommandOutputUtf8

def CheckPacman(package):
    output = GetCommandOutputUtf8(["pacman", "-Q", "--info", package])

    print(output)

    return output.splitlines()[1].split(":", 1)[1].strip()



import subprocess

from libvgazer.exceptions import CommandError

def RunCommand(command, verbose, allowedReturncodes=None):
    if verbose:
        print(" ".join(command))
    try:
        if verbose:
            subprocess.check_call(command)
        else:
            subprocess.check_output(command, stderr=subprocess.STDOUT)
    except subprocess.CalledProcessError as e:
        if (allowedReturncodes is None
         or e.returncode not in allowedReturncodes):
            if not verbose:
                print("Output:", e.output)
            print("VGAZER: Error occured while running utility:", command[0])
            raise CommandError(
             "Error occured while running utility:" + command[0], e.cmd,
             e.returncode)

def GetCommandOutputUtf8(command, verbose=False):
    try:
        if verbose:
            print(" ".join(command))
        output = subprocess.check_output(
         command, stderr=subprocess.STDOUT
        ).decode("utf-8")
        if verbose:
            print(output)
        return output
    except subprocess.CalledProcessError as e:
        if verbose:
            print(e.output.decode("utf-8"))
        raise CommandError("Error occured while running utility:" + command[0],
         e.cmd, e.returncode)



import os

from libvgazer.command     import RunCommand
from libvgazer.exceptions  import CommandError
from libvgazer.exceptions  import InstallError
from libvgazer.platform    import GetInstallPrefix
from libvgazer.store.temp  import StoreTemp
from libvgazer.working_dir import WorkingDir

def Install(software, platform, platformData, verbose):
    installPrefix = GetInstallPrefix(platformData)

    storeTemp = StoreTemp()
    storeTemp.ResolveEmptySubdirectory(software)
    tempPath = storeTemp.GetSubdirectoryPath(software)

    url = ("https://raw.githubusercontent.com/bilke/cmake-modules/master/"
     "CodeCoverage.cmake")

    try:
        with WorkingDir(tempPath):
            RunCommand(
             [
              "git", "clone", "https://github.com/bilke/cmake-modules.git", "."
             ],
             verbose)

            if not os.path.exists(installPrefix + "/lib/cmake/cmake-modules"):
                RunCommand(
                 ["mkdir", "-p", installPrefix + "/lib/cmake/cmake-modules"],
                 verbose)
            RunCommand(
             [
              "cp", "./CodeCoverage.cmake",
              installPrefix + "/lib/cmake/cmake-modules"
             ],
             verbose)
    except CommandError:
        print("VGAZER: Unable to install", software)
        raise InstallError(software + " not installed")

    print("VGAZER:", software, "installed")



import os

from libvgazer.command     import RunCommand
from libvgazer.exceptions  import CommandError
from libvgazer.exceptions  import InstallError
from libvgazer.platform    import GetInstallPrefix
from libvgazer.store.temp  import StoreTemp
from libvgazer.working_dir import WorkingDir

def Install(software, platform, platformData, verbose):
    installPrefix = GetInstallPrefix(platformData)

    storeTemp = StoreTemp()
    storeTemp.ResolveEmptySubdirectory(software)
    tempPath = storeTemp.GetSubdirectoryPath(software)

    try:
        with WorkingDir(tempPath):
            RunCommand(
             ["git", "clone", "https://github.com/attractivechaos/klib.git"],
             verbose)
        clonedDir = os.path.join(tempPath, "klib")
        with WorkingDir(clonedDir):
            if not os.path.exists(
             "{prefix}/include/klib".format(prefix=installPrefix)):
                RunCommand(
                 [
                  "mkdir", "-p",
                  "{prefix}/include/klib".format(prefix=installPrefix)
                 ],
                 verbose)
            RunCommand(
             [
              "cp", "./ketopt.h",
              "{prefix}/include/klib".format(prefix=installPrefix)
             ],
             verbose)
    except CommandError:
        print("VGAZER: Unable to install", software)
        raise InstallError(
         "{software} not installed".format(software=software))

    print("VGAZER:", software, "installed")



from setuptools import find_packages
from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="vgazer",
    version="73",
    url="https://github.com/edomin/vgazer",
    license="CC0",
    author="Vasiliy Edomin",
    author_email="Vasiliy.Edomin@gmail.com",
    description="Library for checking versions and installing various software",
    packages=find_packages(),
    scripts=["vgazer"],
    install_requires=[
        "bs4",
        "multimethod",
        "requests",
        "yolk3k",
    ],
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "License :: CC0 1.0 Universal (CC0 1.0) Public Domain Dedication",
        "Natural Language :: English",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python :: 3",
        "Topic :: Software Development :: Build Tools",
    ],
    zip_safe=False
)



import os

from libvgazer.command     import RunCommand
from libvgazer.exceptions  import CommandError
from libvgazer.exceptions  import InstallError
from libvgazer.platform    import GetInstallPrefix
from libvgazer.platform    import GetTriplet
from libvgazer.store.temp  import StoreTemp
from libvgazer.version.git import GetLastTag
from libvgazer.working_dir import WorkingDir

def Install(software, platform, platformData, verbose):
    installPrefix = GetInstallPrefix(platformData)
    targetTriplet = GetTriplet(platformData["target"])

    storeTemp = StoreTemp()
    storeTemp.ResolveEmptySubdirectory(software)
    tempPath = storeTemp.GetSubdirectoryPath(software)

    try:
        with WorkingDir(tempPath):
            RunCommand(
             ["git", "clone", "https://github.com/libexpat/libexpat.git", "."],
             verbose)
            RunCommand(
             [
              "git", "checkout",
              GetLastTag("https://github.com/libexpat/libexpat.git",
               hint=r'R\_\d\_\d\_\d')
             ],
             verbose)
        expatDir = os.path.join(tempPath, "expat")
        with WorkingDir(expatDir):
            RunCommand(["./buildconf.sh"], verbose)
            RunCommand(
             [
              "./configure", "--host={triplet}".format(triplet=targetTriplet),
              "--prefix={prefix}".format(prefix=installPrefix),
              "--disable-shared", "--without-examples", "--without-docbook"
             ],
             verbose)
            RunCommand(["make", "-j{jobs}".format(jobs=os.cpu_count())],
             verbose, allowedReturncodes=[2])
            RunCommand(["make", "install"], verbose)
    except CommandError:
        print("VGAZER: Unable to install", software)
        raise InstallError(
         "{software} not installed".format(software=software))

    print("VGAZER:", software, "installed")



from libvgazer.platform   import GetAr
from libvgazer.platform   import GetCc
from libvgazer.platform   import GetCxx
from libvgazer.platform   import GetInstallPrefix
from libvgazer.platform   import GetTriplet
from libvgazer.platform   import Platform
from libvgazer.store.temp import StoreTemp

class ConfigCmake():
    def __init__(self, platformData, cflags=None):
        print("VGAZER: Generating CMake toolchain file...")
        
        storeTemp = StoreTemp(".vgazer/cmake")
        self.crossFileName = storeTemp.GetDirectoryFilePath("toolchain.cmake")
        storeTemp.ResolveDirectory()

        cmakeOs = Platform.GetGenericOs(
         platformData["target"].GetOs()).capitalize()
        prefix = GetInstallPrefix(platformData)
        targetTriplet = GetTriplet(platformData["target"])

        cc = GetCc(platformData["target"])
        cxx = GetCxx(platformData["target"])
        ar = GetAr(platformData["target"])

        pkgConfigLibdirs = ("/usr/{triplet}/lib/pkgconfig"
         ":{prefix}/lib/pkgconfig:{prefix}/share/pkgconfig"
         ":/usr/lib/{triplet}/pkgconfig").format(
          prefix=prefix,
          triplet=targetTriplet
         )

        if platformData["target"].IsHost():
            findMode = "BOTH"
            pkgConfigLibdirs += ":/usr/share/pkgconfig:/usr/lib/pkgconfig"
        else:
            findMode = "ONLY"

        storeTemp.DirectoryWriteTextFile("toolchain.cmake",
         "set(CMAKE_SYSTEM_NAME {cmakeOs})\n"
         "set(CMAKE_SYSTEM_PROCESSOR {cpu})\n"
         "set(CMAKE_C_COMPILER {cc})\n"
         "set(CMAKE_C_FLAGS {cflags})\n"
         "set(CMAKE_CXX_COMPILER {cxx})\n"
         "set(CMAKE_AR {ar})\n"
         "set(CMAKE_FIND_ROOT_PATH {prefix})\n"
         "SET("
          "ENV{{PKG_CONFIG_LIBDIR}} "
          "{pkgConfigLibdirs}"
         ")\n"
         "SET(CMAKE_FIND_ROOT_PATH_MODE_PROGRAM NEVER)\n"
         "SET(CMAKE_FIND_ROOT_PATH_MODE_LIBRARY {findMode})\n"
         "SET(CMAKE_FIND_ROOT_PATH_MODE_INCLUDE {findMode})".format(
          cmakeOs=cmakeOs,
          cpu=platformData["target"].GetArch(),
          cc=cc,
          cflags=cflags if cflags is not None else "",
          cxx=cxx,
          ar=ar,
          prefix=prefix,
          pkgConfigLibdirs=pkgConfigLibdirs,
          findMode=findMode
         )
        )

    def __enter__(self):
        return self.crossFileName

    def __exit__(self, etype, value, traceback):
        pass



import os

from libvgazer.command      import GetCommandOutputUtf8
from libvgazer.command      import RunCommand
from libvgazer.exceptions   import CommandError
from libvgazer.exceptions   import InstallError
from libvgazer.store.temp   import StoreTemp
from libvgazer.version.git  import GetLastTag
from libvgazer.working_dir  import WorkingDir

def Install(software, platform, platformData, verbose):
    storeTemp = StoreTemp()
    storeTemp.ResolveEmptySubdirectory(software)
    tempPath = storeTemp.GetSubdirectoryPath(software)

    try:
        output = GetCommandOutputUtf8(["clang", "--version"])
        version = output.splitlines()[0].split(" ")[-1].split(".")[0]

        with WorkingDir(tempPath):
            RunCommand(
             [
              "git", "clone",
              "https://github.com/include-what-you-use/include-what-you-use.git",
              "."
             ],
             verbose)
            RunCommand(
             ["git", "checkout", "clang_{version}".format(version=version)],
             verbose)
            RunCommand(["mkdir", "build"], verbose)
        buildDir = os.path.join(tempPath, "build")
        with WorkingDir(buildDir):
            RunCommand(
             [
              "cmake", "..", "-DCMAKE_INSTALL_PREFIX=/usr/local",
              "-DCMAKE_PREFIX_PATH=/usr", "-DCMAKE_VERBOSE_MAKEFILE:BOOL=ON",
             ],
             verbose)
            RunCommand(
             ["make", "-j{cores_count}".format(cores_count=os.cpu_count())],
             verbose)
            RunCommand(["make", "install"], verbose)
    except CommandError:
        print("VGAZER: Unable to install", software)
        raise InstallError(
         "{software} not installed".format(software=software))

    print("VGAZER:", software, "installed")



import os
import requests
from bs4 import BeautifulSoup

from libvgazer.command      import RunCommand
from libvgazer.config.cmake import ConfigCmake
from libvgazer.exceptions   import CommandError
from libvgazer.exceptions   import InstallError
from libvgazer.platform     import GetArFullPath
from libvgazer.platform     import GetInstallPrefix
from libvgazer.store.temp   import StoreTemp
from libvgazer.version.git  import GetLastTag
from libvgazer.working_dir  import WorkingDir

def Install(software, platform, platformData, verbose):
    installPrefix = GetInstallPrefix(platformData)
    ar = GetArFullPath(platformData["target"])

    storeTemp = StoreTemp()
    storeTemp.ResolveEmptySubdirectory(software)
    tempPath = storeTemp.GetSubdirectoryPath(software)

    try:
        with WorkingDir(tempPath):
            RunCommand(
             [
              "git", "clone", "https://git.cryptomilk.org/projects/cmocka.git",
              "."
             ],
             verbose)
            RunCommand(
             [
              "git", "checkout",
              GetLastTag("https://git.cryptomilk.org/projects/cmocka.git")
             ],
             verbose)
            RunCommand(["mkdir", "build"], verbose)
        buildDir = os.path.join(tempPath, "build")
        with WorkingDir(buildDir), ConfigCmake(platformData) as conf:
            RunCommand(
             [
              "cmake", "..",
              "-DCMAKE_TOOLCHAIN_FILE={conf}".format(conf=conf),
              "-DCMAKE_BUILD_TYPE=Debug",
              "-DCMAKE_INSTALL_PREFIX={prefix}".format(prefix=installPrefix),
              "-DCMAKE_VERBOSE_MAKEFILE:BOOL=ON",
              "-DCMAKE_AR={ar}".format(ar=ar), "-DWITH_STATIC_LIB=ON"
             ],
             verbose)
            RunCommand(
             ["make", "-j{cores_count}".format(cores_count=os.cpu_count())],
             verbose)
            RunCommand(["make", "install"], verbose)
    except CommandError:
        print("VGAZER: Unable to install", software)
        raise InstallError(
         "{software} not installed".format(software=software))

    print("VGAZER:", software, "installed")



import os
import re

from libvgazer.command     import GetCommandOutputUtf8
from libvgazer.command     import RunCommand
from libvgazer.exceptions  import CommandError
from libvgazer.store.temp  import StoreTemp
from libvgazer.working_dir import WorkingDir

def ArchToDebianArch(arch):
    return {
        "x86_64": "amd64",
    }[arch]

def CheckAptCache(package, arch):
    output = GetCommandOutputUtf8(["apt-cache", "madison", package])
    lines = output.splitlines()
    maxVersion = ""

    for line in lines:
        columns = line.split("|")
        version = columns[1].strip()
        repoArch = columns[2].strip().split(" ", 2)[2]
        if ((repoArch == "{arch} Packages".format(arch=ArchToDebianArch(arch))
         or repoArch == "Sources") and (version > maxVersion)):
            maxVersion = version

    return maxVersion



from libvgazer.command     import RunCommand
from libvgazer.exceptions  import CommandError
from libvgazer.exceptions  import InstallError

def InstallPip3(software, package, verbose):
    try:
        RunCommand(["pip3", "install", package, "--no-cache-dir"], verbose)
    except CommandError:
        print("VGAZER: Unable to install", software)
        raise InstallError(software + " not installed")

    print("VGAZER:", software, "installed")



import os
import shutil

from libvgazer.exceptions import DirNameEngaged
from libvgazer.exceptions import FilenameEngaged

class StoreBase():
    def __init__(self, baseDirPath):
        self.baseDirPath = baseDirPath

    def GetDirectoryPath(self):
        return self.baseDirPath

    def GetSubdirectoryPath(self, subdirName):
        return os.path.join(self.baseDirPath, subdirName)

    def GetDirectoryFilePath(self, filename):
        return os.path.join(self.baseDirPath, filename)

    def GetSubdirectoryFilePath(self, subdirName, filename):
        return os.path.join(self.baseDirPath, subdirName, filename)

    def DirectoryExists(self):
        if os.path.exists(self.baseDirPath):
            if os.path.isdir(self.baseDirPath):
                return True
            else:
                raise DirNameEngaged(
                 "Name of directory for storing local settings are engaged "
                 "by file"
                )
        else:
            return False

    def SubdirectoryExists(self, subdirName):
        subdirPath = os.path.join(self.baseDirPath, subdirName)
        if os.path.exists(subdirPath):
            if os.path.isdir(subdirPath):
                return True
            else:
                raise DirNameEngaged(
                 "Name of directory " + subdirPath + " are engaged by file")
        else:
            return False

    def SubdirectoryFileExists(self, subdirName, filename):
        subdirFilePath = os.path.join(self.baseDirPath, subdirName, filename)
        if os.path.exists(subdirFilePath):
            if os.path.isfile(subdirFilePath):
                return True
            else:
                raise FilenameEngaged(
                 subdirFilePath + " is not a file and its filename are engaged"
                )
        else:
            return False

    def DirectoryFileExists(self, filename):
        filePath = os.path.join(self.baseDirPath, filename)
        if os.path.exists(filePath):
            if os.path.isfile(filePath):
                return True
            else:
                raise FilenameEngaged(
                 filePath + " is not a file and its filename are engaged")
        else:
            return False

    def ResolveDirectory(self):
        try:
            if not self.DirectoryExists():
                os.makedirs(self.baseDirPath)
            return True
        except self.DirNameEngaged:
            return False

    def ResolveSubdirectory(self, subdirName):
        self.ResolveDirectory()
        try:
            if not self.SubdirectoryExists(subdirName):
                subdirPath = os.path.join(self.baseDirPath, subdirName)
                os.makedirs(subdirPath)
            return True
        except DirNameEngaged:
            return False

    def ResolveEmptySubdirectory(self, subdirName):
        self.ResolveDirectory()
        try:
            if self.SubdirectoryExists(subdirName):
                self.RemoveSubdirectory(subdirName)
            subdirPath = os.path.join(self.baseDirPath, subdirName)
            os.makedirs(subdirPath)
            return True
        except DirNameEngaged:
            return False

    def ResolveDirectoryFile(self, subdirName, filename):
        self.ResolveDirectory()
        try:
            if not self.DirectoryFileExists(filename):
                filePath = os.path.join(self.baseDirPath, filename)
                open(filePath, "a").close()
            return True
        except FilenameEngaged:
            return False

    def ResolveSubdirectoryFile(self, subdirName, filename):
        self.ResolveDirectory()
        self.ResolveSubdirectory(subdirName)
        try:
            if not self.SubdirectoryFileExists(subdirName, filename):
                subdirFilePath = os.path.join(self.baseDirPath, subdirName,
                 filename)
                open(subdirFilePath, "a").close()
            return True
        except FilenameEngaged:
            return False

    def SubdirectoryFileIsEmpty(self, subdirName, filename):
        subdirFilePath = os.path.join(self.baseDirPath, subdirName, filename)
        if os.stat(subdirFilePath).st_size == 0:
            return True
        return False

    def SubdirectoryReadFile(self, subdirName, filename, mode):
        filepath = os.path.join(self.baseDirPath, subdirName, filename)
        fh = open(filepath, mode)
        data = fh.read()
        fh.close()
        return data

    def SubdirectoryReadTextFile(self, subdirName, filename):
        return self.SubdirectoryReadFile(subdirName, filename, "r")

    def SubdirectoryReadBinaryFile(self, subdirName, filename):
        return self.SubdirectoryReadFile(subdirName, filename, "rb")

    def SubdirectoryWriteFile(self, subdirName, filename, data, mode):
        filepath = os.path.join(self.baseDirPath, subdirName, filename)
        fh = open(filepath, mode)
        fh.write(data)
        fh.close()
        return data

    def SubdirectoryWriteTextFile(self, subdirName, filename, data):
        return self.SubdirectoryWriteFile(subdirName, filename, data, "w")

    def SubdirectoryWriteBinaryFile(self, subdirName, filename, data):
        return self.SubdirectoryWriteFile(subdirName, filename, data, "wb")

    def RemoveSubdirectory(self, subdirName):
        dirpath = os.path.join(self.baseDirPath, subdirName)
        shutil.rmtree(dirpath)

    def DirectoryReadTextFile(self, filename):
        return self.SubdirectoryReadTextFile(".", filename)

    def DirectoryReadBinaryFile(self, filename):
        return self.SubdirectoryReadBinaryFile(".", filename)

    def DirectoryWriteTextFile(self, filename, data):
        return self.SubdirectoryWriteTextFile(".", filename, data)

    def DirectoryWriteBinaryFile(self, filename, data):
        return self.SubdirectoryWriteBinaryFile(".", filename, data)



import os

from libvgazer.command     import RunCommand
from libvgazer.exceptions  import CommandError
from libvgazer.exceptions  import InstallError
from libvgazer.platform    import GetAr
from libvgazer.platform    import GetCc
from libvgazer.platform    import GetInstallPrefix
from libvgazer.store.temp  import StoreTemp
from libvgazer.version.git import GetLastTag
from libvgazer.working_dir import WorkingDir

def Install(software, platform, platformData, verbose):
    installPrefix = GetInstallPrefix(platformData)
    ar = GetAr(platformData["target"])
    cc = GetCc(platformData["target"])

    storeTemp = StoreTemp()
    storeTemp.ResolveEmptySubdirectory(software)
    tempPath = storeTemp.GetSubdirectoryPath(software)

    try:
        with WorkingDir(tempPath):
            RunCommand(
             ["git", "clone", "https://github.com/benhoyt/inih.git", "."],
             verbose)
            RunCommand(
             [
              "git", "checkout",
              GetLastTag("https://github.com/benhoyt/inih.git")
             ],
             verbose)
            RunCommand(
             [
              "sed", "-i",
              "-e", "/#include <stdio.h>/a #define INI_HANDLER_LINENO 1",
              "-e",
              "/#include <stdio.h>"
              "/a #define INI_CALL_HANDLER_ON_NEW_SECTION 1",
              "-e", "/#include <stdio.h>/a #define INI_MAX_LINE 8192",
              "./ini.h",
             ],
             verbose)
            RunCommand([cc, "-c", "ini.c", "-o", "ini.o", "-O2", "-fPIC"],
             verbose)
            RunCommand([ar, "rcs", "libinih.a", "ini.o"], verbose)
            if not os.path.exists(
             "{prefix}/include".format(prefix=installPrefix)):
                RunCommand(
                 [
                  "mkdir", "-p",
                  "{prefix}/include".format(prefix=installPrefix)
                 ],
                 verbose
                )
            if not os.path.exists("{prefix}/lib".format(prefix=installPrefix)):
                RunCommand(
                 ["mkdir", "-p", "{prefix}/lib".format(prefix=installPrefix)],
                 verbose)
            RunCommand(
             ["cp", "./ini.h",
              "{prefix}/include".format(prefix=installPrefix)],
             verbose)
            RunCommand(
             ["cp", "./libinih.a",
              "{prefix}/lib".format(prefix=installPrefix)],
             verbose)
    except CommandError:
        print("VGAZER: Unable to install", software)
        raise InstallError(
         "{software} not installed".format(software=software))

    print("VGAZER:", software, "installed")



data = {
    "patch": {
        "platform": "host",
        "projects": [
            {
                "arch": ["any"],
                "os": ["oraclelinux"],
                "osVersion": ["7"],
                "abi": ["any"],
                "changelog": "https://savannah.gnu.org/news/?group=patch",
                "checker": {
                    "type": "yum",
                    "package": "patch",
                },
                "installer": {
                    "type": "yum",
                    "package": "patch",
                },
            },
        ],
    },
    "perl-data-dumper": {
        "platform": "host",
        "projects": [
            {
                "arch": ["any"],
                "os": ["oraclelinux"],
                "osVersion": ["7"],
                "abi": ["any"],
                "changelog": "https://perldoc.perl.org/perldelta",
                "checker": {
                    "type": "yum",
                    "package": "perl-Data-Dumper",
                },
                "installer": {
                    "type": "yum",
                    "package": "perl-Data-Dumper",
                },
            },
        ],
    },
    "perl-thread-queue": {
        "platform": "host",
        "projects": [
            {
                "arch": ["any"],
                "os": ["oraclelinux"],
                "osVersion": ["7"],
                "abi": ["any"],
                "changelog": "https://metacpan.org/dist/Thread-Queue",
                "checker": {
                    "type": "yum",
                    "package": "perl-Thread-Queue",
                },
                "installer": {
                    "type": "yum",
                    "package": "perl-Thread-Queue",
                },
            },
        ],
    },
    "pkg-config": {
        "platform": "host",
        "projects": [
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["any"],
                "changelog": "https://gitlab.freedesktop.org/pkg-config/pkg-config/-/blob/master/NEWS?ref_type=heads",
                "checker": {
                    "type": "pacman",
                    "package": "pkgconf",
                },
                "installer": {
                    "type": "pacman",
                    "package": "pkgconf",
                },
            },
            {
                "arch": ["any"],
                "os": ["oraclelinux"],
                "osVersion": ["7"],
                "abi": ["any"],
                "changelog": "https://gitlab.freedesktop.org/pkg-config/pkg-config/-/blob/master/NEWS?ref_type=heads",
                "checker": {
                    "type": "yum",
                    "package": "pkgconfig",
                },
                "installer": {
                    "type": "cmd",
                    "cmds": [
                        ["mkdir", "-p", "/usr/local/share/aclocal"],
                        [
                            "ln", "-s", "/usr/share/aclocal/pkg.m4",
                            "/usr/local/share/aclocal/pkg.m4"
                        ],
                    ],
                },
            },
            {
                "arch": ["any"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["any"],
                "changelog": "https://gitlab.freedesktop.org/pkg-config/pkg-config/-/blob/master/NEWS?ref_type=heads",
                "checker": {
                    "type": "apt-cache",
                    "package": "pkg-config",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
        ],
    },
    "pthread-stubs": {
        "platform": "target",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["x11"],
                "changelog": "https://gitlab.freedesktop.org/xorg/lib/pthread-stubs/-/tags",
                "prereqs": [
                    "{triplet}-gcc",
                    "autoconf",
                    "automake",
                    "make",
                ],
                "checker": {
                    "type": "git",
                    "url": "https://gitlab.freedesktop.org/xorg/lib/pthread-stubs.git",
                },
                "installer": {
                    "type": "custom",
                    "name": "pthread-stubs",
                },
            },
            {
                "arch": ["any"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["gnu"],
                "license": ["x11"],
                "changelog": "https://gitlab.freedesktop.org/xorg/lib/pthread-stubs/-/tags",
                "checker": {
                    "type": "apt-cache",
                    "package": "libpthread-stubs0-dev",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
        ],
    },
    "safeclib": {
        "platform": "target",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["mit"],
                "changelog": "https://github.com/rurban/safeclib/blob/master/ChangeLog",
                "prereqs": [
                    "{triplet}-gcc",
                    "autoconf",
                    "automake",
                    "libtool",
                    "make",
                ],
                "checker": {
                    "type": "git",
                    "url": "https://github.com/rurban/safeclib.git",
                    "hint": r'v\d\.\d(\.\d)?$',
                },
                "installer": {
                    "type": "custom",
                    "name": "safeclib",
                },
            },
            # Temporary stub
            {
                "arch": ["any"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["gnu"],
                "license": ["mit"],
                "changelog": "https://gitlab.freedesktop.org/xorg/lib/pthread-stubs/-/tags",
                "checker": {
                    "type": "apt-cache",
                    "package": "libsafec-dev",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
        ],
    },
    "scv": {
        "platform": "target",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["mit"],
                "changelog": "https://github.com/jibsen/scv/commits/master/",
                "prereqs": [
                    "{triplet}-gcc",
                ],
                "checker": {
                    "type": "git",
                    "url": "https://github.com/jibsen/scv.git",
                },
                "installer": {
                    "type": "custom",
                    "name": "scv",
                },
            },
        ],
    },
    "stb_rect_pack": {
        "platform": "target",
        "projects": [
            {
                "fallback": True,
                "arch": ["any"],
                "os": ["any"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["unlicense"],
                "changelog": "https://github.com/nothings/stb/blob/master/stb_rect_pack.h#L42",
                "checker": {
                    "type": "git",
                    "url": "https://github.com/nothings/stb.git",
                    "files": ["stb_rect_pack.h"],
                    "hint": r'\d\.\d{2}',
                },
                "installer": {
                    "type": "custom",
                    "name": "stb_rect_pack",
                },
            },
            {
                "arch": ["any"],
                "os": ["archlinux"],
                "osVersion": ["latest"],
                "abi": ["any"],
                "license": ["unlicense"],
                "changelog": "https://github.com/nothings/stb/blob/master/stb_rect_pack.h#L42",
                "checker": {
                    "type": "pacman",
                    "package": "stb",
                },
                "installer": {
                    "type": "pacman",
                    "package": "stb",
                },
            },
            # Temporary stub
            {
                "arch": ["any"],
                "os": ["steamrt"],
                "osVersion": ["any"],
                "abi": ["any"],
                "license": ["unlicense"],
                "changelog": "https://github.com/nothings/stb/blob/master/stb_rect_pack.h#L42",
                "checker": {
                    "type": "apt-cache",
                    "package": "libstb-dev",
                },
                "installer": {
                    "type": "not-needed",
                },
            },
        ],
    },
}
